@@ -764,10 +764,9 @@ class TestVectorsWycheproof(unittest.TestCase):
         self._extra_params = extra_params
 
     def setUp(self):
-        file_in = open(pycryptodome_filename(
-                        "Cryptodome.SelfTest.Cipher.test_vectors.wycheproof".split("."),
-                        "aes_gcm_test.json"), "rt")
-        tv_tree = json.load(file_in)
+        comps = "Cryptodome.SelfTest.Cipher.test_vectors.wycheproof".split(".")
+        with open(pycryptodome_filename(comps, "aes_gcm_test.json"), "rt") as file_in:
+            tv_tree = json.load(file_in)
 
         class TestVector(object):
             pass
@@ -874,7 +873,7 @@ class TestVariableLength(unittest.TestCase):
 
 
 def get_tests(config={}):
-    from Cryptodome.Util import _cpuid
+    from Cryptodome.Util import _cpu_features
 
     wycheproof_warnings = config.get('wycheproof_warnings')
 
@@ -888,7 +887,7 @@ def get_tests(config={}):
     if config.get('slow_tests'):
         tests += list_test_cases(NISTTestVectorsGCM)
 
-    if _cpuid.have_clmul():
+    if _cpu_features.have_clmul():
         tests += [ TestVectorsWycheproof(wycheproof_warnings, use_clmul=False) ]
         tests += [ TestVariableLength(use_clmul = False) ]
         if config.get('slow_tests'):
